// A library for the cheap but useful 28BYJ-48 5v stepper motor with ULN2003 driver board
//Wrapped by Johan  Schoeman
//Changelog
//20190720: NEW
#pragma once
#include "B4RDefines.h"
#include "AccelStepper.h"
#include "MultiStepper.h"

namespace B4R {
	//~Version: 1.0
	//~shortname: AccelStepper
	class B4RAccelStepper {
		private:
			AccelStepper* bn;
			uint8_t be[sizeof(AccelStepper)];
		public:
	
			/**
			*
			*Initializes the AccelStepper object with the pin numbers to be used
			*If enable is true (the default), enableOutputs() will be called to enable the output pins at construction time.
			*
			*/			
			void Initialize(byte interface, byte pin1, byte pin2, byte pin3, byte pin4, bool enable);
			
			/**
			*
			*Poll the motor and step it if a step is due, implementing
			*accelerations and decelerations to acheive the target position. You must call this as
			*frequently as possible, but at least once per minimum step time interval,
			*preferably in your main loop. Note that each call to run() will make at most one step, and then only when a step is due,
			*based on the current speed and the time since the last step.
			*return true if the motor is still running to the target position.
			*
			*/					
			bool run();
			
			/**
			*
			*Set the target position. The run() function will try to move the motor (at most one step per call)
			*from the current position to the target position set by the most
			*recent call to this function. Caution: moveTo() also recalculates the speed for the next step. 
			*If you are trying to use constant speed movements, you should call setSpeed() after calling moveTo().
			*param[in] absolute -  The desired absolute position. Negative is anticlockwise from the 0 position.	
			*
			*/	
			void moveTo(long absolute); 
			
			/**
			*
			*Sets the desired constant speed for use with runSpeed().
			*param[in] speed The desired constant speed in steps per
			*second. Positive is clockwise. Speeds of more than 1000 steps per
			*second are unreliable. Very slow speeds may be set (eg 0.00027777 for
			*once per hour, approximately. Speed accuracy depends on the Arduino
			*crystal. Jitter depends on how frequently you call the runSpeed() function.	
			*
			*/		
			void setSpeed(float speed);	
			
			/**
			*
			*Runs at the currently selected speed until the target position is reached
			*Does not implement accelerations.
			*return true if it stepped	
			*
			*/			
			bool runSpeedToPosition();	
			
			/**
			*
			*Poll the motor and step it if a step is due, implementing a constant
			*speed as set by the most recent call to setSpeed(). You must call this as
			*frequently as possible, but at least once per step interval,
			*return true if the motor was stepped.		
			*
			*/			
			bool runSpeed();	
			
			/**
			*
			*Sets the maximum permitted speed. The run() function will accelerate
			*up to the speed set by this function.
			*Caution: the maximum speed achievable depends on your processor and clock speed.
			*param[in] speed The desired maximum speed in steps per second. Must
			*be > 0. Caution: Speeds that exceed the maximum speed supported by the processor may
			*Result in non-linear accelerations and decelerations.
			*
			*/				
			void setMaxSpeed(float speed);	

			/**
			*
			*Set the target position relative to the current position
			*param[in] relative The desired position relative to the current position. Negative is
			*anticlockwise from the current position.
			*
			*/					
			void move(long relative);

			/**
			*
			*returns the maximum speed configured for this stepper
			*that was previously set by setMaxSpeed();
			*return The currently configured maximum speed
			*
			*/						
			float maximumSpeed();	

			/**
			*
			*Sets the acceleration/deceleration rate.
			*param[in] acceleration The desired acceleration in steps per second
			*per second. Must be > 0.0. This is an expensive call since it requires a square 
			*root to be calculated. Dont call more ofthen than needed
			*
			*/				
			void setAcceleration(float acceleration);	

			/**
			*
			*The most recently set speed
			*return the most recent speed in steps per second
			*
			*/			
			float GetSpeed();

			/**
			*
			*The distance from the current position to the target position.
			*return the distance from the current position to the target position
			*in steps. Positive is clockwise from the current position.
			*
			*/			
			long distanceToGo();

			/**
			*
			*The most recently set target position.
			*return the target position
			*in steps. Positive is clockwise from the 0 position.
			*
			*/		
			long targetPosition();

			/**
			*
			*The currently motor position.
			*return the current motor position
			*in steps. Positive is clockwise from the 0 position.
			*
			*/			
			long GetCurrentPosition();  
			
			/**
			*
			*Resets the current position of the motor, so that wherever the motor
			*happens to be right now is considered to be the new 0 position. Useful
			*for setting a zero position on a stepper after an initial hardware
			*positioning move.
			*Has the side effect of setting the current motor speed to 0.
			*param[in] position The position in steps of wherever the motor
			*happens to be right now.
			*
			*/				
			void setCurrentPosition(long position);  
    
			/**
			*
			*Moves the motor (with acceleration/deceleration) 
			*to the target position and blocks until it is at
			*position. Dont use this in event loops, since it blocks.
			*
			*/			
			void runToPosition();	

			/**
			*
			*Moves the motor (with acceleration/deceleration)
			*to the new target position and blocks until it is at
			*position. Dont use this in event loops, since it blocks.
			*param[in] position The new target position.
			*
			*/				
			void runToNewPosition(long position);

			/**
			*
			*Sets a new target position that causes the stepper
			*to stop as quickly as possible, using the current speed and acceleration parameters.
			*
			*/			
			void stop();

			/**
			*
			*Disable motor pin outputs by setting them all LOW
			*Depending on the design of your electronics this may turn off
			*the power to the motor coils, saving power.
			*This is useful to support Arduino low power modes: disable the outputs
			*during sleep and then reenable with enableOutputs() before stepping
			*again.
			*If the enable Pin is defined, sets it to OUTPUT mode and clears the pin to disabled.
			*
			*/			
			void disableOutputs();

			/**
			*
			*Enable motor pin outputs by setting the motor pins to OUTPUT
			*mode. Called automatically by the constructor.
			*If the enable Pin is defined, sets it to OUTPUT mode and sets the pin to enabled.
			*
			*/			
			void enableOutputs();

			/**
			*
			*Sets the minimum pulse width allowed by the stepper driver. The minimum practical pulse width is 
			*approximately 20 microseconds. Times less than 20 microseconds
			*will usually result in 20 microseconds or so.
			*param[in] minWidth The minimum pulse width in microseconds. 
			*
			*/			
			void setMinPulseWidth(UInt minWidth);

			/**
			*
			*Sets the enable pin number for stepper drivers.
			*0xFF indicates unused (default).
			*Otherwise, if a pin is set, the pin will be turned on when 
			*enableOutputs() is called and switched off when disableOutputs() 
			*is called.
			*param[in] enablePin Arduino digital pin number for motor enable
			*sa setPinsInverted
			*
			*/			
			void setEnablePin(byte enablePin = 0xff);
	
			/**
			*
			*Sets the inversion for stepper driver pins
			*param[in] directionInvert True for inverted direction pin, false for non-inverted
			*param[in] stepInvert      True for inverted step pin, false for non-inverted
			*param[in] enableInvert    True for inverted enable pin, false (default) for non-inverted
			*
			*/			
			void setPinsInverted_1(bool directionInvert, bool stepInvert, bool enableInvert);
	
			/**
			*
			*Sets the inversion for 2, 3 and 4 wire stepper pins
			*param[in] pin1Invert True for inverted pin1, false for non-inverted
			*param[in] pin2Invert True for inverted pin2, false for non-inverted
			*param[in] pin3Invert True for inverted pin3, false for non-inverted
			*param[in] pin4Invert True for inverted pin4, false for non-inverted
			*param[in] enableInvert    True for inverted enable pin, false (default) for non-inverted
			*
			*/				
			void setPinsInverted_2(bool pin1Invert, bool pin2Invert, bool pin3Invert, bool pin4Invert, bool enableInvert);

			/**
			*
			*Checks to see if the motor is currently running to a target
			*return true if the speed is not zero or not at the target position
			*
			*/			
			bool isRunning();	
			
			/**
			*
			*2 wire stepper, 2 motor pins required
			*
			*/						
			byte FULL2WIRE = 2;
			/**
			*
			*3 wire stepper, such as HDD spindle, 3 motor pins required
			*
			*/				
			byte FULL3WIRE = 3;
			/**
			*
			*4 wire full stepper, 4 motor pins required
			*
			*/						
			byte FULL4WIRE = 4;
			/**
			*
			*3 wire half stepper, such as HDD spindle, 3 motor pins required
			*
			*/						
			byte HALF3WIRE = 6;
			/**
			*
			*4 wire half stepper, 4 motor pins required
			*
			*/					
			byte HALF4WIRE = 8;


			
			
	};
	
	
	
	
	
	
/* 	//~shortname: MultiStepper	
	class B4RMultiStepper {
		private:
			MultiStepper* bo;
			uint8_t bf[sizeof(MultiStepper)];
		public: */
	
			/**
			*
			*Initializes the AccelStepper object with the pin numbers to be used
			*If enable is true (the default), enableOutputs() will be called to enable the output pins at construction time.
			*
			*/			
//			void Initialize();
			
//			bool addStepper(B4RAccelStepper& stepper);
			

			
			
//	};
	

}